// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Stage.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ACTIVATEAPP()
	ON_WM_PALETTECHANGED()
	ON_WM_MOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pScene = NULL;
	m_pPlane = NULL;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// Create the 3D window
	if (!m_wnd3d.Create(this, IDC_3DWND)) {
		return -1;
	}

	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	SetScene();
	ASSERT(m_pScene);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// Make the initial window a fixed size
	cs.cx = 300;
	cs.cy = 350;

	return CFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::RecalcLayout(BOOL bNotify) 
{
    // Rearrange the control bars and fit the 3D window in the middle
	// Let the frame rearrange the control bars
	CFrameWnd::RecalcLayout(bNotify);

	// Find the space that's left over
	CRect rc;
	RepositionBars(0,
				   0xFFFF,
				   IDC_3DWND,
				   CWnd::reposQuery,
				   &rc);
	if (IsWindow(m_wnd3d.GetSafeHwnd())) {
		m_wnd3d.MoveWindow(&rc, FALSE);
	}
}


void CMainFrame::OnDestroy() 
{
	CFrameWnd::OnDestroy();
	
	// Delete any scene we might have
	m_wnd3d.SetScene(NULL);
	if (m_pScene) {
		delete m_pScene;
		m_pScene = NULL;
	}
}

void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask) 
{
	CFrameWnd::OnActivateApp(bActive, hTask);
	
	// Tell the 3D window about the new state
	m_wnd3d.SendMessage(WM_ACTIVATEAPP,
						(WPARAM)bActive,
						(LPARAM)hTask);
}

void CMainFrame::OnPaletteChanged(CWnd* pFocusWnd) 
{
	// Let the 3D window know
	m_wnd3d.SendMessage(WM_PALETTECHANGED,
					   pFocusWnd ? (WPARAM)pFocusWnd->GetSafeHwnd() : 0);
}

void CMainFrame::OnMove(int x, int y) 
{
	CFrameWnd::OnMove(x, y);
	
	// Let the 3D window know the frame has moved
	m_wnd3d.SendMessage(WM_MOVE, 0, MAKELPARAM(0, 0));
}

BOOL CMainFrame::SetScene()
{
	// Create an initial scene
	m_pScene = new C3dScene;
	if (!m_pScene->Create()) return FALSE;
	m_wnd3d.SetScene(m_pScene);

	// Set up the lighting
    C3dDirLight dl;
    dl.Create(0.8, 0.4, 0.4);
    m_pScene->AddChild(&dl);
    dl.SetPosition(-2, 2, -5);
    dl.SetDirection(1, -1, 1);
    m_pScene->SetAmbientLight(0.4, 0.4, 0.4);

	// Build the scenery
	C3dShape* pLand = new C3dShape();

    D3DVECTOR vlist[] = {
        {-10.0,  8.0,  10.0}, //  0
        { -2.0,  6.0,  10.0}, //  1
        {  2.0,  8.0,  10.0}, //  2
        {  8.0,  6.0,  10.0}, //  3
		{-10.0,  6.0,   2.0}, //  4
		{ -5.0,  0.0,	4.0}, //  5
		{ -3.0,  0.0,   5.0}, //  6
		{ -1.0,  0.0,   4.0}, //  7
		{  2.0,  0.0,   4.0}, //  8
		{  4.0,  0.0,   5.0}, //  9
		{  5.0,  0.0,   3.0}, // 10
		{ 10.0,  9.0,   4.0}, // 11
		{ -6.0,  0.0,   0.0}, // 12
		{ -4.0,  0.0,   1.0}, // 13
		{ -3.0,  0.0,   2.5}, // 14
		{  0.0,  8.0,   0.0}, // 15
		{ -4.0,  0.0,  -1.0}, // 16
		{ -3.0,  0.0,  -2.0}, // 17
		{  3.0,  0.0,  -1.0}, // 18
		{  3.0,  0.0,  -2.0}, // 19
		{  5.0,  0.0,  -3.0}, // 20
		{ 10.0,  6.0,  -3.0}, // 21
		{-10.0,  8.0,  -6.0}, // 22
		{ -5.0,  0.0,  -4.0}, // 23
		{ -4.0,  0.0,  -5.0}, // 24
		{ -1.0,  0.0,  -4.0}, // 25
		{  2.0,  0.0,  -6.0}, // 26
		{  3.0,  0.0,  -5.0}, // 27
		{ -9.0,  6.0, -10.0}, // 28
		{ -4.0,  8.0, -10.0}, // 29
		{  3.0,  6.0, -10.0}, // 30
		{ 10.0,  8.0, -10.0}, // 31
		
		{  0.0,  0.0,   0.0}  // EOL
    };
    int iVectors = sizeof(vlist) / sizeof(D3DVECTOR);
    int iFaces[] = {3, 0, 5, 4, // 0  
                    3, 0, 6, 5,	// 1
					3, 0, 1, 6, // 2
					3, 1, 7, 6, // 3
					3, 1, 2, 7, // 4
					3, 7, 2, 8, // 5
					3, 8, 2, 9, // 6
					3, 9, 2, 3, // 7
					3, 9, 3, 10, // 8
					3, 10, 3, 11, // 9
					3, 4, 5, 13, // 10
					3, 13, 5, 14, // 11
					3, 14, 5, 6, // 12
					3, 15, 14, 6, // 13
					3, 15, 6, 7, // 14
					3, 15, 7, 8, // 15
					3, 15, 8, 9, // 17
					3, 15, 9, 20, // 18
					3, 15, 10, 18, // 19
					3, 18, 10, 21, // 20
					3, 21, 10, 11, // 21
					3, 22, 4, 12, // 22
					3, 12, 4, 13, // 23
					3, 12, 13, 15, // 24
					3, 13, 14, 15, // 25
					3, 16, 12, 15, // 26
					3, 17, 16, 15, // 27
					3, 24, 17, 15, // 28
					3, 24, 15, 25, // 29
					3, 25, 15, 26, // 30
					3, 26, 15, 27, // 31
					3, 27, 15, 19, // 32
					3, 19, 15, 18, // 33
					3, 19, 18, 21, // 34
					3, 22, 12, 16, // 35
					3, 22, 16, 17, // 36
					3, 22, 17, 23, // 37
					3, 23, 17, 24, // 38
					3, 27, 19, 20, // 39
					3, 20, 19, 21, // 40
					3, 28, 22, 23, // 41
					3, 28, 23, 29, // 42
					3, 29, 23, 24, // 43
					3, 29, 24, 25, // 44
					3, 29, 25, 30, // 45
					3, 30, 25, 26, // 46
					3, 30, 26, 27, // 47
					3, 30, 27, 31, // 48
					3, 31, 27, 20, // 49
					3, 31, 20, 21, // 50

                    0};

    pLand->Create(vlist, iVectors, iFaces);

	C3dTexture* pRocks = new C3dTexture;
	pRocks->Load(IDB_ROCKS);
	m_pScene->m_ImgList.Append(pRocks);

	// Create a flat wrap for the rocks
	C3dWrap wrap;
	wrap.Create(D3DRMWRAP_FLAT,
				 NULL,
				 0, 0, 0, // origin
				 0, 1, 0, // direction (y)
				 0, 0, 1, // up	(z)
				 0, 0,	 // texture origin
				 2, 2);	 // texture scale
	pLand->SetTexture(pRocks);
	wrap.Apply(pLand);

	m_pScene->AddChild(pLand);
	m_pScene->m_ShapeList.Append(pLand);
	m_pScene->SetBackground(0, 0, 0.5);

	// Set up the camera flight path
	m_vCamera = C3dVector(5, 5, 0);
	m_dRadius = 5.0;

	// Set field of view
	m_pScene->SetCameraField(1.5);

	// Make an airplane to follow
    D3DVECTOR vlistP[] = {
        {-0.5,  0.0, -0.3},
        { 0.0,  0.1, -0.3},
        { 0.5,  0.0, -0.3},
        { 0.0,  0.0,  0.7},
        {-0.1,  0.0, -0.3},
        { 0.1,  0.0, -0.3},
        { 0.0,  0.3, -0.3},
        { 0.0,  0.0,  0.1}
    };
    int iPVectors = sizeof(vlistP) / sizeof(D3DVECTOR);
    int iPFaces[] = {3, 0, 3, 1, // 0 left wing top
                     3, 1, 3, 2, // 1 right wing top
                     3, 2, 3, 0, // 2 wing underside
                     3, 0, 1, 2, // 3 wing aft face
                     3, 4, 6, 5, // 4 tail aft face
                     3, 5, 6, 7, // 5 tail right side
                     3, 7, 6, 4, // 6 tail left side
                     0};

	m_pPlane = new C3dShape;
	m_pPlane->Create(vlistP, iPVectors, iPFaces);
	m_pPlane->SetFaceColor(0, 0, 1, 0);
	m_pPlane->SetFaceColor(1, 0, 1, 0);
	m_pPlane->SetFaceColor(3, 1, 0, 0);
	m_pPlane->SetFaceColor(4, 1, 0, 0);
	m_pScene->AddChild(m_pPlane);
	m_pScene->m_ShapeList.Append(m_pPlane);

	return TRUE;
}

BOOL CMainFrame::Update(double d)
{
	// Update camera position
	C3dMatrix r;
	r.Rotate(0, 2.0, 0);
	m_vCamera = r * m_vCamera;

	m_pScene->SetCameraPosition(m_vCamera);

	// Set the up vector
	C3dVector vu(0, 1, 0);

	// Generate forward vector
	C3dVector vf = m_vCamera * vu;
	m_pScene->SetCameraDirection(vf, vu);

	// Set airplane position in front of camera
	r.Rotate(0, 20, 0);
	C3dVector vp = r * m_vCamera;
	m_pPlane->SetPosition(vp);

	// Set direction
	C3dMatrix rp;
	rp.Rotate(0, 0, 10); // A bit wobbly!
	vu = rp * vu;
	vf = vp * vu;
	m_pPlane->SetDirection(vf, vu);

	return m_wnd3d.Update(TRUE);
}